package com.massa.mrules.droolsfight.tests;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import com.massa.mrules.droolsfight.beans.MeasureBean;
import com.massa.mrules.droolsfight.beans.MeasureData;
import com.massa.mrules.droolsfight.beans.MeasureError;
import com.massa.mrules.droolsfight.beans.SampleData;

public class PrepareBeans {

	public static List<MeasureBean> getAllBeans() {
		final List<MeasureBean> ret = new ArrayList<MeasureBean>();

		ret.add(PrepareBeans.passingGood());
		ret.add(PrepareBeans.passingMedium());
		ret.add(PrepareBeans.passingBad());
		ret.add(PrepareBeans.datesKo1());
		ret.add(PrepareBeans.datesKo2());
		ret.add(PrepareBeans.locationKo1());
		ret.add(PrepareBeans.locationKo2());
		ret.add(PrepareBeans.contactsKo1());
		ret.add(PrepareBeans.contactsKo2());
		ret.add(PrepareBeans.samplesKo1());
		ret.add(PrepareBeans.samplesKo2());
		ret.add(PrepareBeans.samplesKo3());
		ret.add(PrepareBeans.samplesKo4());
		ret.add(PrepareBeans.samplesKo5());
		ret.add(PrepareBeans.multipleKo1());
		ret.add(PrepareBeans.multipleKo2());

		return ret;
	}


	private static MeasureBean testBase() {
		final MeasureData md = new MeasureData();

		final Calendar c = Calendar.getInstance();
		md.setEndDate(c.getTime());
		c.add(Calendar.DAY_OF_MONTH, -7);
		md.setStartDate(c.getTime());

		md.setLocation("Address and Town");

		md.setContacts(new HashSet<String>());
		md.getContacts().add("addr1@domain1.ext");
		md.getContacts().add("addr2@domain2.ext");

		md.setNumberOfSamples(5);
		md.setFrequency("H");

		md.setSampleData(new HashMap<String, SampleData>());
		md.getSampleData().put("NO2", new SampleData("NO2", Double.valueOf(0.0035)));
		md.getSampleData().put("CO2", new SampleData("CO2", Double.valueOf(0.00015)));

		final MeasureBean mb = new MeasureBean(md);
		mb.setExpectedNbBlocking(0);
		mb.setExpectedAirQuality("GOOD");

		return mb;
	}

	public static MeasureBean passingGood() {
		final MeasureBean mb = PrepareBeans.testBase();

		return mb;
	}

	public static MeasureBean passingMedium() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().getSampleData().put("PM10", new SampleData("PM10", Double.valueOf(0.012)));
		mb.setExpectedAirQuality("MEDIUM");
		return mb;
	}

	public static MeasureBean passingBad() {
		final MeasureBean mb = PrepareBeans.passingMedium();
		mb.getMeasure().getSampleData().put("SO2", new SampleData("SO2", Double.valueOf(0.043)));
		mb.setExpectedAirQuality("BAD");
		return mb;
	}

	public static MeasureBean datesKo1() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setEndDate(null);
		mb.addExpectedError(new MeasureError("DatesOk", "Dates are not present.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean datesKo2() {
		final MeasureBean mb = PrepareBeans.testBase();
		final Calendar c = Calendar.getInstance();
		c.add(Calendar.DAY_OF_MONTH, +7);
		mb.getMeasure().setStartDate(c.getTime());
		mb.addExpectedError(new MeasureError("DatesOk", "End date before start date.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean locationKo1() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setLocation(null);
		mb.addExpectedError(new MeasureError("LocationOk", "Location is not present.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean locationKo2() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setLocation("");
		mb.addExpectedError(new MeasureError("LocationOk", "Location is not present.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean contactsKo1() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setContacts(null);
		mb.addExpectedError(new MeasureError("ContactsOk", "Contacts are not present.", false));
		return mb;
	}

	public static MeasureBean contactsKo2() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().getContacts().add("incorrect.email.address");
		mb.addExpectedError(new MeasureError("ContactsOk", "Invalid contact.", false));
		return mb;
	}

	public static MeasureBean samplesKo1() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setNumberOfSamples(-2);
		mb.addExpectedError(new MeasureError("SamplesOk", "Incorrect number of samples.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean samplesKo2() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setNumberOfSamples(15);
		mb.addExpectedError(new MeasureError("SamplesOk", "Incorrect number of samples.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean samplesKo3() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setFrequency(null);
		mb.addExpectedError(new MeasureError("SamplesOk", "Incorrect frequency value.", false));
		return mb;
	}

	public static MeasureBean samplesKo4() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().setFrequency("T");
		mb.addExpectedError(new MeasureError("SamplesOk", "Incorrect frequency value.", false));
		return mb;
	}

	public static MeasureBean samplesKo5() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().getSampleData().clear();
		mb.addExpectedError(new MeasureError("SamplesOk", "Between 1 and 5 pollutants can be measured.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean samplesKo6() {
		final MeasureBean mb = PrepareBeans.testBase();
		mb.getMeasure().getSampleData().put("INCORRECT", new SampleData("INCORRECT", 1.0D));
		mb.addExpectedError(new MeasureError("SamplesOk", "Incorrect pollutant type.", true));
		mb.setExpectedNbBlocking(1);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean multipleKo1() {
		final MeasureBean mb = PrepareBeans.locationKo2();
		mb.getMeasure().getSampleData().put("INCORRECT", new SampleData("INCORRECT", 1.0D));
		mb.addExpectedError(new MeasureError("SamplesOk", "Incorrect pollutant type.", true));
		mb.setExpectedNbBlocking(2);
		mb.setExpectedAirQuality(null);
		return mb;
	}

	public static MeasureBean multipleKo2() {
		final MeasureBean mb = PrepareBeans.multipleKo1();
		mb.getMeasure().getContacts().add("incorrect.email.address");
		mb.addExpectedError(new MeasureError("ContactsOk", "Invalid contact.", false));
		mb.setExpectedNbBlocking(2);
		mb.setExpectedAirQuality(null);
		return mb;
	}

}
